package lightcontainer.protocol.rules.writer;

import lightcontainer.domains.client.Context;
import lightcontainer.protocol.ProtocolWriter;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;

/**
 * Règle signifiant que la sauvegarde d'un fichier fût un succès
 */
public class SaveFileOkRule extends ProtocolWriter {

    private static final String PATTERN = "^SAVEFILE_OK\r\n$";

    public static final String NAME = "SAVEFILE_OK";

    private final String storagePath;

    public SaveFileOkRule(String storagePath) {
        super(NAME, PATTERN);
        this.storagePath = storagePath;
    }

    @Override
    protected ProtocolWriter.ProtocolResult onExecuted(Context context, String... data) {
        ProtocolWriter.ProtocolResult result = new ProtocolWriter.ProtocolResult(context);

        // Sauvegarder dans JSON
        context.addFile(context.getDataString("fileName"), context.getDataString("fileNameSalt"), context.getDataInt("size"), context.getDataString("iv"), context.getDomain());

        // Suppression du fichier temporaire dans le stockage du FFE
        String hashedFileName = context.getHashedFileName(context.getDataString("fileName"));
        try {
            Files.deleteIfExists(Path.of(String.format("%s/%s", storagePath, hashedFileName)));
        } catch (IOException ignore) {
        }
        return result;
    }
}
