package lightcontainer.repository;

import lightcontainer.domains.client.Context;
import lightcontainer.domains.client.StoreProcessor;
import lightcontainer.domains.Task;
import lightcontainer.enumerations.TaskStatus;
import lightcontainer.interfaces.ClientHandlerFFE;
import lightcontainer.interfaces.ProtocolRepository;
import lightcontainer.interfaces.StoreProcessorFFE;
import lightcontainer.protocol.ProtocolWriter;

import java.util.Deque;
import java.util.Iterator;
import java.util.concurrent.ConcurrentLinkedDeque;

public class FileFrontEnd implements ClientHandlerFFE, StoreProcessorFFE {
    // Variables
    private Deque<Task> tasks = new ConcurrentLinkedDeque<>();
    private final ClientHandlerRepository clientRepository; // TODO -> pourquoi pas une interface ? end
    private final StoreProcessorRepository storeRepository; // TODO -> pourquoi pas une interface ? end
    private ProtocolRepository protocolRepository;

    // Constructor
    public FileFrontEnd(ClientHandlerRepository clientRepo, StoreProcessorRepository storeRepo, ProtocolRepository protocolRepository) {
        this.clientRepository = clientRepo;
        this.storeRepository = storeRepo;
        this.protocolRepository = protocolRepository;
    }

    /**
     * Appelé quand nouvelle tâche
     */
    public void alertStoreProcessors(Task task) {
        // On avertit les stor processors d'une nouvelle tâche
        String stor = storeRepository.findDomain(task);
        if (stor != null) {
            storeRepository.assignTask(stor, task);
            task.setDomain(stor);
        }
    }

    /**
     * Permet à un {@link StoreProcessor} d'avertir le FFE qu'il est disponible
     *
     * @param store Le SBE qui s'est occupé de la tâche
     * @param response La réponse à envoyer au client
     */
    @Override
    public void onStoreAvailable(StoreProcessor store, ProtocolWriter.ProtocolResult response) {
        // TODO : Chercher une tâche appropriée
        Iterator<Task> it = tasks.iterator();
        while (it.hasNext()) {
            Task task = it.next();
            System.out.println("Cherche");
            if (task.isResponseOfClient(store.getDomain())) {
                System.out.println("Task trouvée");
                clientRepository.respondToClient(task.getClient(), response);
                it.remove(); // Suppression de la tâche
                break;
            }
        }

        assignOtherTask(store);
    }

    @Override
    public void onStoreDisconnect(String domain) {
        this.storeRepository.closeStore(domain);
    }

    private void assignOtherTask(StoreProcessor store) {
        Iterator<Task> it = tasks.iterator();

        while (it.hasNext()) {
            Task task = it.next();
            if (store.canProcessTask(task)) {
                storeRepository.assignTask(store.getDomain(), task);
                task.setDomain(store.getDomain());
                return;
            }
        }
    }


    @Override
    public void newCommand(Context context, ProtocolWriter.ProtocolResult command) {
        Task task = Task.newInstance(context, command);
        tasks.add(task);
        alertStoreProcessors(task);
    }

    @Override
    public boolean canExecuteCommand(String domain) {
        return domain == null ? storeRepository.domainCount() > 0 : storeRepository.hasDomain(domain);
    }
}
